"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .email import Email, EmailTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class LinkedEcommerceCustomerTypedDict(TypedDict):
    r"""The customer this entity is linked to."""

    id: NotRequired[Nullable[str]]
    r"""The ID of the customer this entity is linked to."""
    name: NotRequired[Nullable[str]]
    r"""Full name of the customer"""
    first_name: NotRequired[Nullable[str]]
    r"""First name of the customer"""
    last_name: NotRequired[Nullable[str]]
    r"""Last name of the customer"""
    company_name: NotRequired[Nullable[str]]
    r"""Company name of the customer"""
    phone_numbers: NotRequired[Nullable[List[PhoneNumberTypedDict]]]
    emails: NotRequired[Nullable[List[EmailTypedDict]]]


class LinkedEcommerceCustomer(BaseModel):
    r"""The customer this entity is linked to."""

    id: OptionalNullable[str] = UNSET
    r"""The ID of the customer this entity is linked to."""

    name: OptionalNullable[str] = UNSET
    r"""Full name of the customer"""

    first_name: OptionalNullable[str] = UNSET
    r"""First name of the customer"""

    last_name: OptionalNullable[str] = UNSET
    r"""Last name of the customer"""

    company_name: OptionalNullable[str] = UNSET
    r"""Company name of the customer"""

    phone_numbers: OptionalNullable[List[PhoneNumber]] = UNSET

    emails: OptionalNullable[List[Email]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "company_name",
            "phone_numbers",
            "emails",
        ]
        nullable_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "company_name",
            "phone_numbers",
            "emails",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class LinkedAttachmentTypedDict(TypedDict):
    name: NotRequired[Nullable[str]]
    r"""The name of the file"""
    mime_type: NotRequired[Nullable[str]]
    r"""The MIME type of the file."""
    is_compressed: NotRequired[Nullable[bool]]
    r"""Whether the file is c ompressed."""
    encoding: NotRequired[Nullable[str]]
    r"""The encoding of the file."""
    content: NotRequired[Nullable[str]]
    r"""The content of the file."""
    notes: NotRequired[Nullable[str]]
    r"""The notes of the file."""


class LinkedAttachment(BaseModel):
    name: OptionalNullable[str] = UNSET
    r"""The name of the file"""

    mime_type: OptionalNullable[str] = UNSET
    r"""The MIME type of the file."""

    is_compressed: OptionalNullable[bool] = UNSET
    r"""Whether the file is c ompressed."""

    encoding: OptionalNullable[str] = UNSET
    r"""The encoding of the file."""

    content: OptionalNullable[str] = UNSET
    r"""The content of the file."""

    notes: OptionalNullable[str] = UNSET
    r"""The notes of the file."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "mime_type",
            "is_compressed",
            "encoding",
            "content",
            "notes",
        ]
        nullable_fields = [
            "name",
            "mime_type",
            "is_compressed",
            "encoding",
            "content",
            "notes",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.models import ApideckError
from apideck_unify.types import BaseModel
import httpx
from typing import Any, Dict, Optional, Union
from typing_extensions import TypeAliasType


BadRequestResponseDetailTypedDict = TypeAliasType(
    "BadRequestResponseDetailTypedDict", Union[str, Dict[str, Any]]
)
r"""Contains parameter or domain specific information related to the error and why it occurred."""


BadRequestResponseDetail = TypeAliasType(
    "BadRequestResponseDetail", Union[str, Dict[str, Any]]
)
r"""Contains parameter or domain specific information related to the error and why it occurred."""


class BadRequestResponseData(BaseModel):
    status_code: Optional[float] = None
    r"""HTTP status code"""

    error: Optional[str] = None
    r"""Contains an explanation of the status_code as defined in HTTP/1.1 standard (RFC 7231)"""

    type_name: Optional[str] = None
    r"""The type of error returned"""

    message: Optional[str] = None
    r"""A human-readable message providing more details about the error."""

    detail: Optional[BadRequestResponseDetail] = None
    r"""Contains parameter or domain specific information related to the error and why it occurred."""

    ref: Optional[str] = None
    r"""Link to documentation of error type"""


class BadRequestResponse(ApideckError):
    r"""Bad Request"""

    data: BadRequestResponseData

    def __init__(
        self,
        data: BadRequestResponseData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SuppliersFilterTypedDict(TypedDict):
    company_name: NotRequired[str]
    r"""Company Name of supplier to search for"""
    display_name: NotRequired[str]
    r"""Display Name of supplier to search for"""
    first_name: NotRequired[str]
    r"""First name of supplier to search for"""
    last_name: NotRequired[str]
    r"""Last name of supplier to search for"""
    email: NotRequired[str]
    r"""Email of supplier to search for"""
    updated_since: NotRequired[datetime]


class SuppliersFilter(BaseModel):
    company_name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Company Name of supplier to search for"""

    display_name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Display Name of supplier to search for"""

    first_name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""First name of supplier to search for"""

    last_name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Last name of supplier to search for"""

    email: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Email of supplier to search for"""

    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentStatus(str, Enum):
    r"""Status of payment"""

    DRAFT = "draft"
    AUTHORISED = "authorised"
    REJECTED = "rejected"
    PAID = "paid"
    VOIDED = "voided"
    DELETED = "deleted"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EcommerceOrderStatus(str, Enum):
    r"""Current status of the order."""

    ACTIVE = "active"
    COMPLETED = "completed"
    CANCELLED = "cancelled"
    ARCHIVED = "archived"
    UNKNOWN = "unknown"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InvoicesFilterTypedDict(TypedDict):
    updated_since: NotRequired[datetime]
    created_since: NotRequired[datetime]
    number: NotRequired[str]
    r"""Invoice number to search for"""


class InvoicesFilter(BaseModel):
    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None

    created_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None

    number: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Invoice number to search for"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .consumermetadata import ConsumerMetadata, ConsumerMetadataTypedDict
from .unifiedapiid import UnifiedAPIID
from apideck_unify.types import BaseModel
from enum import Enum
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class AllowActions(str, Enum):
    DELETE = "delete"
    DISCONNECT = "disconnect"
    REAUTHORIZE = "reauthorize"
    DISABLE = "disable"


class SettingsTypedDict(TypedDict):
    r"""Settings to change the way the Vault is displayed."""

    unified_apis: NotRequired[List[UnifiedAPIID]]
    r"""Provide the IDs of the Unified APIs you want to be visible. Leaving it empty or omitting this field will show all Unified APIs."""
    hide_resource_settings: NotRequired[bool]
    r"""A boolean that controls the display of the configurable resources for an integration. When set to true, the resource configuration options will be hidden and not shown to the user. When set to false, the resource configuration options will be displayed to the user."""
    sandbox_mode: NotRequired[bool]
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show a banner informing the logged in user is in a test environment."""
    isolation_mode: NotRequired[bool]
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to run in isolation mode, meaning it only shows the connection settings and hides the navigation items."""
    session_length: NotRequired[str]
    r"""The duration of time the session is valid for (maximum 1 week)."""
    show_logs: NotRequired[bool]
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the logs page. Defaults to `true`."""
    show_suggestions: NotRequired[bool]
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the suggestions page. Defaults to `false`."""
    show_sidebar: NotRequired[bool]
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the sidebar. Defaults to `true`."""
    auto_redirect: NotRequired[bool]
    r"""Automatically redirect to redirect uri after the connection has been configured as callable. Defaults to `false`."""
    hide_guides: NotRequired[bool]
    r"""Hide Apideck connection guides in [Vault](/apis/vault/reference#section/Get-Started). Defaults to `false`."""
    allow_actions: NotRequired[List[AllowActions]]
    r"""Hide actions from your users in [Vault](/apis/vault/reference#section/Get-Started). Actions in `allow_actions` will be shown on a connection in Vault.
    Available actions are: `delete`, `disconnect`, `reauthorize` and `disable`.
    Empty array will hide all actions. By default all actions are visible.
    """


class Settings(BaseModel):
    r"""Settings to change the way the Vault is displayed."""

    unified_apis: Optional[List[UnifiedAPIID]] = None
    r"""Provide the IDs of the Unified APIs you want to be visible. Leaving it empty or omitting this field will show all Unified APIs."""

    hide_resource_settings: Optional[bool] = False
    r"""A boolean that controls the display of the configurable resources for an integration. When set to true, the resource configuration options will be hidden and not shown to the user. When set to false, the resource configuration options will be displayed to the user."""

    sandbox_mode: Optional[bool] = False
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show a banner informing the logged in user is in a test environment."""

    isolation_mode: Optional[bool] = False
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to run in isolation mode, meaning it only shows the connection settings and hides the navigation items."""

    session_length: Optional[str] = "1h"
    r"""The duration of time the session is valid for (maximum 1 week)."""

    show_logs: Optional[bool] = True
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the logs page. Defaults to `true`."""

    show_suggestions: Optional[bool] = False
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the suggestions page. Defaults to `false`."""

    show_sidebar: Optional[bool] = True
    r"""Configure [Vault](/apis/vault/reference#section/Get-Started) to show the sidebar. Defaults to `true`."""

    auto_redirect: Optional[bool] = False
    r"""Automatically redirect to redirect uri after the connection has been configured as callable. Defaults to `false`."""

    hide_guides: Optional[bool] = False
    r"""Hide Apideck connection guides in [Vault](/apis/vault/reference#section/Get-Started). Defaults to `false`."""

    allow_actions: Optional[List[AllowActions]] = None
    r"""Hide actions from your users in [Vault](/apis/vault/reference#section/Get-Started). Actions in `allow_actions` will be shown on a connection in Vault.
    Available actions are: `delete`, `disconnect`, `reauthorize` and `disable`.
    Empty array will hide all actions. By default all actions are visible.
    """


class ThemeTypedDict(TypedDict):
    r"""Theming options to change the look and feel of Vault."""

    favicon: NotRequired[str]
    r"""The URL to the favicon to use for Vault."""
    logo: NotRequired[str]
    r"""The URL to the logo to use for Vault."""
    primary_color: NotRequired[str]
    r"""The primary color to use for Vault."""
    sidepanel_background_color: NotRequired[str]
    r"""The background color to use for the sidebar."""
    sidepanel_text_color: NotRequired[str]
    r"""The text color to use for the sidebar."""
    vault_name: NotRequired[str]
    r"""The name that will be shown in the sidebar."""
    privacy_url: NotRequired[str]
    r"""The URL to the privacy policy that will be shown in the sidebar."""
    terms_url: NotRequired[str]
    r"""The URL to the terms and conditions that will be shown in the sidebar."""


class Theme(BaseModel):
    r"""Theming options to change the look and feel of Vault."""

    favicon: Optional[str] = None
    r"""The URL to the favicon to use for Vault."""

    logo: Optional[str] = None
    r"""The URL to the logo to use for Vault."""

    primary_color: Optional[str] = None
    r"""The primary color to use for Vault."""

    sidepanel_background_color: Optional[str] = None
    r"""The background color to use for the sidebar."""

    sidepanel_text_color: Optional[str] = None
    r"""The text color to use for the sidebar."""

    vault_name: Optional[str] = None
    r"""The name that will be shown in the sidebar."""

    privacy_url: Optional[str] = None
    r"""The URL to the privacy policy that will be shown in the sidebar."""

    terms_url: Optional[str] = None
    r"""The URL to the terms and conditions that will be shown in the sidebar."""


class SessionTypedDict(TypedDict):
    consumer_metadata: NotRequired[ConsumerMetadataTypedDict]
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""
    redirect_uri: NotRequired[str]
    r"""The URL to redirect the user to after the session has been configured."""
    settings: NotRequired[SettingsTypedDict]
    r"""Settings to change the way the Vault is displayed."""
    theme: NotRequired[ThemeTypedDict]
    r"""Theming options to change the look and feel of Vault."""
    custom_consumer_settings: NotRequired[Dict[str, Any]]
    r"""Custom consumer settings that are passed as part of the session."""


class Session(BaseModel):
    consumer_metadata: Optional[ConsumerMetadata] = None
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""

    redirect_uri: Optional[str] = None
    r"""The URL to redirect the user to after the session has been configured."""

    settings: Optional[Settings] = None
    r"""Settings to change the way the Vault is displayed."""

    theme: Optional[Theme] = None
    r"""Theming options to change the look and feel of Vault."""

    custom_consumer_settings: Optional[Dict[str, Any]] = None
    r"""Custom consumer settings that are passed as part of the session."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .linkedcustomer import LinkedCustomer, LinkedCustomerTypedDict
from .linkedcustomer_input import LinkedCustomerInput, LinkedCustomerInputTypedDict
from .linkedtrackingcategory import (
    LinkedTrackingCategory,
    LinkedTrackingCategoryTypedDict,
)
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .quotelineitem import (
    QuoteLineItem,
    QuoteLineItemInput,
    QuoteLineItemInputTypedDict,
    QuoteLineItemTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class QuoteStatus(str, Enum):
    r"""Quote status"""

    DRAFT = "draft"
    SENT = "sent"
    ACCEPTED = "accepted"
    REJECTED = "rejected"
    EXPIRED = "expired"
    CONVERTED = "converted"
    VOID = "void"
    DELETED = "deleted"


class QuoteTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    number: NotRequired[Nullable[str]]
    r"""Quote number."""
    customer: NotRequired[Nullable[LinkedCustomerTypedDict]]
    r"""The customer this entity is linked to."""
    invoice_id: NotRequired[str]
    r"""The unique identifier for the invoice."""
    sales_order_id: NotRequired[Nullable[str]]
    r"""The unique identifier for the sales order."""
    company_id: NotRequired[Nullable[str]]
    r"""The company ID the transaction belongs to"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    project_id: NotRequired[str]
    r"""The unique identifier for the linked project."""
    quote_date: NotRequired[Nullable[date]]
    r"""Date quote was issued - YYYY-MM-DD."""
    expiry_date: NotRequired[Nullable[date]]
    r"""The date until which the quote is valid - YYYY-MM-DD."""
    terms: NotRequired[Nullable[str]]
    r"""Terms of the quote."""
    reference: NotRequired[Nullable[str]]
    r"""Optional reference identifier for the transaction."""
    status: NotRequired[Nullable[QuoteStatus]]
    r"""Quote status"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    tax_inclusive: NotRequired[Nullable[bool]]
    r"""Amounts are including tax"""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_tax: NotRequired[Nullable[float]]
    r"""Total tax amount applied to this quote."""
    tax_code: NotRequired[Nullable[str]]
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to this quote."""
    discount_amount: NotRequired[Nullable[float]]
    r"""Discount amount applied to this quote."""
    total: NotRequired[Nullable[float]]
    r"""Total amount of quote, including tax."""
    customer_memo: NotRequired[Nullable[str]]
    r"""Customer memo"""
    line_items: NotRequired[List[QuoteLineItemTypedDict]]
    billing_address: NotRequired[AddressTypedDict]
    shipping_address: NotRequired[AddressTypedDict]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    template_id: NotRequired[Nullable[str]]
    r"""Optional quote template"""
    source_document_url: NotRequired[Nullable[str]]
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Quote(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    number: OptionalNullable[str] = UNSET
    r"""Quote number."""

    customer: OptionalNullable[LinkedCustomer] = UNSET
    r"""The customer this entity is linked to."""

    invoice_id: Optional[str] = None
    r"""The unique identifier for the invoice."""

    sales_order_id: OptionalNullable[str] = UNSET
    r"""The unique identifier for the sales order."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company ID the transaction belongs to"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    project_id: Optional[str] = None
    r"""The unique identifier for the linked project."""

    quote_date: OptionalNullable[date] = UNSET
    r"""Date quote was issued - YYYY-MM-DD."""

    expiry_date: OptionalNullable[date] = UNSET
    r"""The date until which the quote is valid - YYYY-MM-DD."""

    terms: OptionalNullable[str] = UNSET
    r"""Terms of the quote."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional reference identifier for the transaction."""

    status: OptionalNullable[QuoteStatus] = UNSET
    r"""Quote status"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    tax_inclusive: OptionalNullable[bool] = UNSET
    r"""Amounts are including tax"""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_tax: OptionalNullable[float] = UNSET
    r"""Total tax amount applied to this quote."""

    tax_code: OptionalNullable[str] = UNSET
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to this quote."""

    discount_amount: OptionalNullable[float] = UNSET
    r"""Discount amount applied to this quote."""

    total: OptionalNullable[float] = UNSET
    r"""Total amount of quote, including tax."""

    customer_memo: OptionalNullable[str] = UNSET
    r"""Customer memo"""

    line_items: Optional[List[QuoteLineItem]] = None

    billing_address: Optional[Address] = None

    shipping_address: Optional[Address] = None

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    template_id: OptionalNullable[str] = UNSET
    r"""Optional quote template"""

    source_document_url: OptionalNullable[str] = UNSET
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "downstream_id",
            "number",
            "customer",
            "invoice_id",
            "sales_order_id",
            "company_id",
            "department_id",
            "project_id",
            "quote_date",
            "expiry_date",
            "terms",
            "reference",
            "status",
            "currency",
            "currency_rate",
            "tax_inclusive",
            "sub_total",
            "total_tax",
            "tax_code",
            "discount_percentage",
            "discount_amount",
            "total",
            "customer_memo",
            "line_items",
            "billing_address",
            "shipping_address",
            "tracking_categories",
            "template_id",
            "source_document_url",
            "custom_fields",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "downstream_id",
            "number",
            "customer",
            "sales_order_id",
            "company_id",
            "department_id",
            "quote_date",
            "expiry_date",
            "terms",
            "reference",
            "status",
            "currency",
            "currency_rate",
            "tax_inclusive",
            "sub_total",
            "total_tax",
            "tax_code",
            "discount_percentage",
            "discount_amount",
            "total",
            "customer_memo",
            "tracking_categories",
            "template_id",
            "source_document_url",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class QuoteInputTypedDict(TypedDict):
    number: NotRequired[Nullable[str]]
    r"""Quote number."""
    customer: NotRequired[Nullable[LinkedCustomerInputTypedDict]]
    r"""The customer this entity is linked to."""
    sales_order_id: NotRequired[Nullable[str]]
    r"""The unique identifier for the sales order."""
    company_id: NotRequired[Nullable[str]]
    r"""The company ID the transaction belongs to"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    project_id: NotRequired[str]
    r"""The unique identifier for the linked project."""
    quote_date: NotRequired[Nullable[date]]
    r"""Date quote was issued - YYYY-MM-DD."""
    expiry_date: NotRequired[Nullable[date]]
    r"""The date until which the quote is valid - YYYY-MM-DD."""
    terms: NotRequired[Nullable[str]]
    r"""Terms of the quote."""
    reference: NotRequired[Nullable[str]]
    r"""Optional reference identifier for the transaction."""
    status: NotRequired[Nullable[QuoteStatus]]
    r"""Quote status"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    tax_inclusive: NotRequired[Nullable[bool]]
    r"""Amounts are including tax"""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_tax: NotRequired[Nullable[float]]
    r"""Total tax amount applied to this quote."""
    tax_code: NotRequired[Nullable[str]]
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""
    discount_percentage: NotRequired[Nullable[float]]
    r"""Discount percentage applied to this quote."""
    discount_amount: NotRequired[Nullable[float]]
    r"""Discount amount applied to this quote."""
    total: NotRequired[Nullable[float]]
    r"""Total amount of quote, including tax."""
    customer_memo: NotRequired[Nullable[str]]
    r"""Customer memo"""
    line_items: NotRequired[List[QuoteLineItemInputTypedDict]]
    billing_address: NotRequired[AddressTypedDict]
    shipping_address: NotRequired[AddressTypedDict]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    template_id: NotRequired[Nullable[str]]
    r"""Optional quote template"""
    source_document_url: NotRequired[Nullable[str]]
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class QuoteInput(BaseModel):
    number: OptionalNullable[str] = UNSET
    r"""Quote number."""

    customer: OptionalNullable[LinkedCustomerInput] = UNSET
    r"""The customer this entity is linked to."""

    sales_order_id: OptionalNullable[str] = UNSET
    r"""The unique identifier for the sales order."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company ID the transaction belongs to"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    project_id: Optional[str] = None
    r"""The unique identifier for the linked project."""

    quote_date: OptionalNullable[date] = UNSET
    r"""Date quote was issued - YYYY-MM-DD."""

    expiry_date: OptionalNullable[date] = UNSET
    r"""The date until which the quote is valid - YYYY-MM-DD."""

    terms: OptionalNullable[str] = UNSET
    r"""Terms of the quote."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional reference identifier for the transaction."""

    status: OptionalNullable[QuoteStatus] = UNSET
    r"""Quote status"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    tax_inclusive: OptionalNullable[bool] = UNSET
    r"""Amounts are including tax"""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_tax: OptionalNullable[float] = UNSET
    r"""Total tax amount applied to this quote."""

    tax_code: OptionalNullable[str] = UNSET
    r"""Applicable tax id/code override if tax is not supplied on a line item basis."""

    discount_percentage: OptionalNullable[float] = UNSET
    r"""Discount percentage applied to this quote."""

    discount_amount: OptionalNullable[float] = UNSET
    r"""Discount amount applied to this quote."""

    total: OptionalNullable[float] = UNSET
    r"""Total amount of quote, including tax."""

    customer_memo: OptionalNullable[str] = UNSET
    r"""Customer memo"""

    line_items: Optional[List[QuoteLineItemInput]] = None

    billing_address: Optional[Address] = None

    shipping_address: Optional[Address] = None

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    template_id: OptionalNullable[str] = UNSET
    r"""Optional quote template"""

    source_document_url: OptionalNullable[str] = UNSET
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "number",
            "customer",
            "sales_order_id",
            "company_id",
            "department_id",
            "project_id",
            "quote_date",
            "expiry_date",
            "terms",
            "reference",
            "status",
            "currency",
            "currency_rate",
            "tax_inclusive",
            "sub_total",
            "total_tax",
            "tax_code",
            "discount_percentage",
            "discount_amount",
            "total",
            "customer_memo",
            "line_items",
            "billing_address",
            "shipping_address",
            "tracking_categories",
            "template_id",
            "source_document_url",
            "custom_fields",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "number",
            "customer",
            "sales_order_id",
            "company_id",
            "department_id",
            "quote_date",
            "expiry_date",
            "terms",
            "reference",
            "status",
            "currency",
            "currency_rate",
            "tax_inclusive",
            "sub_total",
            "total_tax",
            "tax_code",
            "discount_percentage",
            "discount_amount",
            "total",
            "customer_memo",
            "tracking_categories",
            "template_id",
            "source_document_url",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .consumerconnection import ConsumerConnection, ConsumerConnectionTypedDict
from .consumermetadata import ConsumerMetadata, ConsumerMetadataTypedDict
from .requestcountallocation import (
    RequestCountAllocation,
    RequestCountAllocationTypedDict,
)
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ConsumerTypedDict(TypedDict):
    consumer_id: str
    r"""Unique consumer identifier. You can freely choose a consumer ID yourself. Most of the time, this is an ID of your internal data model that represents a user or account in your system (for example account:12345). If the consumer doesn't exist yet, Vault will upsert a consumer based on your ID."""
    application_id: NotRequired[str]
    r"""ID of your Apideck Application"""
    metadata: NotRequired[ConsumerMetadataTypedDict]
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""
    connections: NotRequired[List[ConsumerConnectionTypedDict]]
    services: NotRequired[List[str]]
    aggregated_request_count: NotRequired[float]
    request_counts: NotRequired[RequestCountAllocationTypedDict]
    created: NotRequired[str]
    modified: NotRequired[str]
    request_count_updated: NotRequired[str]


class Consumer(BaseModel):
    consumer_id: str
    r"""Unique consumer identifier. You can freely choose a consumer ID yourself. Most of the time, this is an ID of your internal data model that represents a user or account in your system (for example account:12345). If the consumer doesn't exist yet, Vault will upsert a consumer based on your ID."""

    application_id: Optional[str] = None
    r"""ID of your Apideck Application"""

    metadata: Optional[ConsumerMetadata] = None
    r"""The metadata of the consumer. This is used to display the consumer in the sidebar. This is optional, but recommended."""

    connections: Optional[List[ConsumerConnection]] = None

    services: Optional[List[str]] = None

    aggregated_request_count: Optional[float] = None

    request_counts: Optional[RequestCountAllocation] = None

    created: Optional[str] = None

    modified: Optional[str] = None

    request_count_updated: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CreditOrDebit(str, Enum):
    r"""Whether the amount is a credit or debit."""

    CREDIT = "credit"
    DEBIT = "debit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class BankAccountType(str, Enum):
    r"""Type of the bank account."""

    BANK = "bank"
    CREDIT_CARD = "credit_card"


class FeedStatus(str, Enum):
    r"""Current status of the bank feed."""

    PENDING = "pending"
    REJECTED = "rejected"


class BankFeedAccountTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    bank_account_type: NotRequired[BankAccountType]
    r"""Type of the bank account."""
    source_account_id: NotRequired[str]
    r"""The source account's unique identifier."""
    target_account_id: NotRequired[str]
    r"""The target account's unique identifier in the accounting connector."""
    target_account_name: NotRequired[str]
    r"""Name associated with the target account."""
    target_account_number: NotRequired[str]
    r"""Account number of the destination bank account."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    feed_status: NotRequired[FeedStatus]
    r"""Current status of the bank feed."""
    country: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""


class BankFeedAccount(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    bank_account_type: Optional[BankAccountType] = None
    r"""Type of the bank account."""

    source_account_id: Optional[str] = None
    r"""The source account's unique identifier."""

    target_account_id: Optional[str] = None
    r"""The target account's unique identifier in the accounting connector."""

    target_account_name: Optional[str] = None
    r"""Name associated with the target account."""

    target_account_number: Optional[str] = None
    r"""Account number of the destination bank account."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    feed_status: Optional[FeedStatus] = None
    r"""Current status of the bank feed."""

    country: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    custom_fields: Optional[List[CustomField]] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "bank_account_type",
            "source_account_id",
            "target_account_id",
            "target_account_name",
            "target_account_number",
            "currency",
            "feed_status",
            "country",
            "custom_fields",
            "custom_mappings",
            "created_at",
            "updated_at",
            "updated_by",
            "created_by",
        ]
        nullable_fields = [
            "currency",
            "country",
            "custom_mappings",
            "created_at",
            "updated_at",
            "updated_by",
            "created_by",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BankFeedAccountInputTypedDict(TypedDict):
    bank_account_type: NotRequired[BankAccountType]
    r"""Type of the bank account."""
    source_account_id: NotRequired[str]
    r"""The source account's unique identifier."""
    target_account_id: NotRequired[str]
    r"""The target account's unique identifier in the accounting connector."""
    target_account_name: NotRequired[str]
    r"""Name associated with the target account."""
    target_account_number: NotRequired[str]
    r"""Account number of the destination bank account."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    feed_status: NotRequired[FeedStatus]
    r"""Current status of the bank feed."""
    country: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]


class BankFeedAccountInput(BaseModel):
    bank_account_type: Optional[BankAccountType] = None
    r"""Type of the bank account."""

    source_account_id: Optional[str] = None
    r"""The source account's unique identifier."""

    target_account_id: Optional[str] = None
    r"""The target account's unique identifier in the accounting connector."""

    target_account_name: Optional[str] = None
    r"""Name associated with the target account."""

    target_account_number: Optional[str] = None
    r"""Account number of the destination bank account."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    feed_status: Optional[FeedStatus] = None
    r"""Current status of the bank feed."""

    country: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    custom_fields: Optional[List[CustomField]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "bank_account_type",
            "source_account_id",
            "target_account_id",
            "target_account_name",
            "target_account_number",
            "currency",
            "feed_status",
            "country",
            "custom_fields",
        ]
        nullable_fields = ["currency", "country"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CustomObjectFieldsTypedDict(TypedDict):
    name: NotRequired[str]
    value: NotRequired[str]


class CustomObjectFields(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None


class CustomObjectTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The unique identifier of the custom object"""
    owner_id: NotRequired[str]
    r"""The unique identifier of the owner of the custom object"""
    name: NotRequired[Nullable[str]]
    r"""The name of the custom object"""
    fields: NotRequired[List[CustomObjectFieldsTypedDict]]
    updated_by: NotRequired[str]
    r"""The unique identifier of the user who last updated the custom object"""
    created_by: NotRequired[str]
    r"""The unique identifier of the user who created the custom object"""
    updated_at: NotRequired[Nullable[str]]
    r"""The timestamp when the custom object was last updated"""
    created_at: NotRequired[Nullable[str]]
    r"""The timestamp when the custom object was created"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CustomObject(BaseModel):
    id: Optional[str] = None
    r"""The unique identifier of the custom object"""

    owner_id: Optional[str] = None
    r"""The unique identifier of the owner of the custom object"""

    name: OptionalNullable[str] = UNSET
    r"""The name of the custom object"""

    fields: Optional[List[CustomObjectFields]] = None

    updated_by: Optional[str] = None
    r"""The unique identifier of the user who last updated the custom object"""

    created_by: Optional[str] = None
    r"""The unique identifier of the user who created the custom object"""

    updated_at: OptionalNullable[str] = UNSET
    r"""The timestamp when the custom object was last updated"""

    created_at: OptionalNullable[str] = UNSET
    r"""The timestamp when the custom object was created"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "owner_id",
            "name",
            "fields",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = ["name", "updated_at", "created_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomObjectInputTypedDict(TypedDict):
    name: NotRequired[Nullable[str]]
    r"""The name of the custom object"""
    fields: NotRequired[List[CustomObjectFieldsTypedDict]]
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CustomObjectInput(BaseModel):
    name: OptionalNullable[str] = UNSET
    r"""The name of the custom object"""

    fields: Optional[List[CustomObjectFields]] = None

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["name", "fields", "pass_through"]
        nullable_fields = ["name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.eventlogs import EventLogs
from apideck_unify.webhooks import Webhooks


class WebhookSDK(BaseSDK):
    webhooks: Webhooks
    event_logs: EventLogs

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.webhooks = Webhooks(self.sdk_configuration)
        self.event_logs = EventLogs(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .compensation import Compensation, CompensationTypedDict
from .payrolltotals import PayrollTotals, PayrollTotalsTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PayrollTypedDict(TypedDict):
    id: Nullable[str]
    r"""A unique identifier for an object."""
    processed: Nullable[bool]
    r"""Whether or not the payroll has been successfully processed. Note that processed payrolls cannot be updated."""
    check_date: Nullable[str]
    r"""The date on which employees will be paid for the payroll."""
    start_date: Nullable[str]
    r"""The start date, inclusive, of the pay period."""
    end_date: Nullable[str]
    r"""The end date, inclusive, of the pay period."""
    company_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the company."""
    processed_date: NotRequired[Nullable[str]]
    r"""The date the payroll was processed."""
    totals: NotRequired[PayrollTotalsTypedDict]
    r"""The overview of the payroll totals."""
    compensations: NotRequired[List[CompensationTypedDict]]
    r"""An array of compensations for the payroll."""
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""


class Payroll(BaseModel):
    id: Nullable[str]
    r"""A unique identifier for an object."""

    processed: Nullable[bool]
    r"""Whether or not the payroll has been successfully processed. Note that processed payrolls cannot be updated."""

    check_date: Nullable[str]
    r"""The date on which employees will be paid for the payroll."""

    start_date: Nullable[str]
    r"""The start date, inclusive, of the pay period."""

    end_date: Nullable[str]
    r"""The end date, inclusive, of the pay period."""

    company_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of the company."""

    processed_date: OptionalNullable[str] = UNSET
    r"""The date the payroll was processed."""

    totals: Optional[PayrollTotals] = None
    r"""The overview of the payroll totals."""

    compensations: Optional[List[Compensation]] = None
    r"""An array of compensations for the payroll."""

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "company_id",
            "processed_date",
            "totals",
            "compensations",
            "custom_mappings",
        ]
        nullable_fields = [
            "id",
            "company_id",
            "processed",
            "processed_date",
            "check_date",
            "start_date",
            "end_date",
            "custom_mappings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .department import Department, DepartmentTypedDict
from .jobstatus import JobStatus
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Visibility(str, Enum):
    r"""The visibility of the job"""

    DRAFT = "draft"
    PUBLIC = "public"
    INTERNAL = "internal"


class EmploymentTerms(str, Enum):
    FULL_TIME = "full-time"
    PART_TIME = "part-time"
    INTERNSHIP = "internship"
    CONTRACTOR = "contractor"
    EMPLOYEE = "employee"
    FREELANCE = "freelance"
    TEMP = "temp"
    SEASONAL = "seasonal"
    VOLUNTEER = "volunteer"
    OTHER = "other"


class BranchTypedDict(TypedDict):
    r"""Details of the branch for which the job is created."""

    id: NotRequired[str]
    r"""A unique identifier for an object."""
    name: NotRequired[str]
    r"""Name of the branch."""


class Branch(BaseModel):
    r"""Details of the branch for which the job is created."""

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    name: Optional[str] = None
    r"""Name of the branch."""


class BlocksTypedDict(TypedDict):
    title: NotRequired[str]
    content: NotRequired[Nullable[str]]


class Blocks(BaseModel):
    title: Optional[str] = None

    content: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["title", "content"]
        nullable_fields = ["content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SalaryTypedDict(TypedDict):
    min: NotRequired[int]
    r"""Minimum salary payable for the job role."""
    max: NotRequired[int]
    r"""Maximum salary payable for the job role."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    interval: NotRequired[Nullable[str]]


class Salary(BaseModel):
    min: Optional[int] = None
    r"""Minimum salary payable for the job role."""

    max: Optional[int] = None
    r"""Maximum salary payable for the job role."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    interval: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["min", "max", "currency", "interval"]
        nullable_fields = ["currency", "interval"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class JobType(str, Enum):
    JOB_PORTAL = "job_portal"
    JOB_DESCRIPTION = "job_description"


class JobLinksTypedDict(TypedDict):
    type: NotRequired[JobType]
    url: NotRequired[str]


class JobLinks(BaseModel):
    type: Optional[JobType] = None

    url: Optional[str] = None


class JobTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    slug: NotRequired[Nullable[str]]
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    sequence: NotRequired[int]
    r"""Sequence in relation to other jobs."""
    visibility: NotRequired[Visibility]
    r"""The visibility of the job"""
    status: NotRequired[JobStatus]
    r"""The status of the job."""
    code: NotRequired[str]
    r"""The code of the job."""
    language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    employment_terms: NotRequired[Nullable[EmploymentTerms]]
    experience: NotRequired[str]
    r"""Level of experience required for the job role."""
    location: NotRequired[Nullable[str]]
    r"""Specifies the location for the job posting."""
    remote: NotRequired[Nullable[bool]]
    r"""Specifies whether the posting is for a remote job."""
    requisition_id: NotRequired[str]
    r"""A job's Requisition ID (Req ID) allows your organization to identify and track a job based on alphanumeric naming conventions unique to your company's internal processes."""
    department: NotRequired[DepartmentTypedDict]
    branch: NotRequired[BranchTypedDict]
    r"""Details of the branch for which the job is created."""
    recruiters: NotRequired[Nullable[List[str]]]
    r"""The recruiter is generally someone who is tasked to help the hiring manager find and screen qualified applicant"""
    hiring_managers: NotRequired[List[str]]
    followers: NotRequired[Nullable[List[str]]]
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    description_html: NotRequired[Nullable[str]]
    r"""The job description in HTML format"""
    blocks: NotRequired[List[BlocksTypedDict]]
    closing: NotRequired[Nullable[str]]
    closing_html: NotRequired[Nullable[str]]
    r"""The closing section of the job description in HTML format"""
    closing_date: NotRequired[Nullable[date]]
    salary: NotRequired[SalaryTypedDict]
    url: NotRequired[Nullable[str]]
    r"""URL of the job description"""
    job_portal_url: NotRequired[Nullable[str]]
    r"""URL of the job portal"""
    record_url: NotRequired[Nullable[str]]
    links: NotRequired[List[JobLinksTypedDict]]
    confidential: NotRequired[bool]
    available_to_employees: NotRequired[bool]
    r"""Specifies whether an employee of the organization can apply for the job."""
    tags: NotRequired[Nullable[List[str]]]
    addresses: NotRequired[List[AddressTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    owner_id: NotRequired[Nullable[str]]
    published_at: NotRequired[Nullable[datetime]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class Job(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    slug: OptionalNullable[str] = UNSET

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    sequence: Optional[int] = None
    r"""Sequence in relation to other jobs."""

    visibility: Optional[Visibility] = None
    r"""The visibility of the job"""

    status: Optional[JobStatus] = None
    r"""The status of the job."""

    code: Optional[str] = None
    r"""The code of the job."""

    language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    employment_terms: OptionalNullable[EmploymentTerms] = UNSET

    experience: Optional[str] = None
    r"""Level of experience required for the job role."""

    location: OptionalNullable[str] = UNSET
    r"""Specifies the location for the job posting."""

    remote: OptionalNullable[bool] = UNSET
    r"""Specifies whether the posting is for a remote job."""

    requisition_id: Optional[str] = None
    r"""A job's Requisition ID (Req ID) allows your organization to identify and track a job based on alphanumeric naming conventions unique to your company's internal processes."""

    department: Optional[Department] = None

    branch: Optional[Branch] = None
    r"""Details of the branch for which the job is created."""

    recruiters: OptionalNullable[List[str]] = UNSET
    r"""The recruiter is generally someone who is tasked to help the hiring manager find and screen qualified applicant"""

    hiring_managers: Optional[List[str]] = None

    followers: OptionalNullable[List[str]] = UNSET

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    description_html: OptionalNullable[str] = UNSET
    r"""The job description in HTML format"""

    blocks: Optional[List[Blocks]] = None

    closing: OptionalNullable[str] = UNSET

    closing_html: OptionalNullable[str] = UNSET
    r"""The closing section of the job description in HTML format"""

    closing_date: OptionalNullable[date] = UNSET

    salary: Optional[Salary] = None

    url: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""URL of the job description"""

    job_portal_url: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""URL of the job portal"""

    record_url: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET

    links: Optional[List[JobLinks]] = None

    confidential: Optional[bool] = None

    available_to_employees: Optional[bool] = None
    r"""Specifies whether an employee of the organization can apply for the job."""

    tags: OptionalNullable[List[str]] = UNSET

    addresses: Optional[List[Address]] = None

    custom_fields: Optional[List[CustomField]] = None

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    owner_id: OptionalNullable[str] = UNSET

    published_at: OptionalNullable[datetime] = UNSET

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "slug",
            "title",
            "sequence",
            "visibility",
            "status",
            "code",
            "language",
            "employment_terms",
            "experience",
            "location",
            "remote",
            "requisition_id",
            "department",
            "branch",
            "recruiters",
            "hiring_managers",
            "followers",
            "description",
            "description_html",
            "blocks",
            "closing",
            "closing_html",
            "closing_date",
            "salary",
            "url",
            "job_portal_url",
            "record_url",
            "links",
            "confidential",
            "available_to_employees",
            "tags",
            "addresses",
            "custom_fields",
            "deleted",
            "owner_id",
            "published_at",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "slug",
            "title",
            "language",
            "employment_terms",
            "location",
            "remote",
            "recruiters",
            "followers",
            "description",
            "description_html",
            "closing",
            "closing_html",
            "closing_date",
            "url",
            "job_portal_url",
            "record_url",
            "tags",
            "deleted",
            "owner_id",
            "published_at",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class FilesSearchTypedDict(TypedDict):
    query: str
    r"""The query to search for. May match across multiple fields."""
    drive_id: NotRequired[str]
    r"""ID of the drive to filter on"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class FilesSearch(BaseModel):
    query: str
    r"""The query to search for. May match across multiple fields."""

    drive_id: Optional[str] = None
    r"""ID of the drive to filter on"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssuesFilterTypedDict(TypedDict):
    status: NotRequired[List[str]]
    r"""Filter by ticket status, can be `open`, `closed` or `all`. Will passthrough if none of the above match"""
    since: NotRequired[datetime]
    r"""Only return tickets since a specific date"""
    assignee_id: NotRequired[str]
    r"""Only return tickets assigned to a specific user"""


class IssuesFilter(BaseModel):
    status: Annotated[Optional[List[str]], FieldMetadata(query=True)] = None
    r"""Filter by ticket status, can be `open`, `closed` or `all`. Will passthrough if none of the above match"""

    since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None
    r"""Only return tickets since a specific date"""

    assignee_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Only return tickets assigned to a specific user"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from enum import Enum
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CategoriesFilterType(str, Enum):
    r"""The type of the category."""

    SUPPLIER = "supplier"
    EXPENSE = "expense"
    REVENUE = "revenue"


class CategoriesFilterTypedDict(TypedDict):
    type: NotRequired[CategoriesFilterType]
    r"""The type of the category."""


class CategoriesFilter(BaseModel):
    type: Annotated[Optional[CategoriesFilterType], FieldMetadata(query=True)] = None
    r"""The type of the category."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .email import Email, EmailTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .taxrate import TaxRate, TaxRateTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class CompanyStatus(str, Enum):
    r"""Based on the status some functionality is enabled or disabled."""

    ACTIVE = "active"
    INACTIVE = "inactive"


class TheStartMonthOfFiscalYear(str, Enum):
    r"""The start month of fiscal year."""

    JANUARY = "January"
    FEBRUARY = "February"
    MARCH = "March"
    APRIL = "April"
    MAY = "May"
    JUNE = "June"
    JULY = "July"
    AUGUST = "August"
    SEPTEMBER = "September"
    OCTOBER = "October"
    NOVEMBER = "November"
    DECEMBER = "December"


class CompanyInfoTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    status: NotRequired[CompanyStatus]
    r"""Based on the status some functionality is enabled or disabled."""
    legal_name: NotRequired[str]
    r"""The legal name of the company"""
    country: NotRequired[Nullable[str]]
    r"""country code according to ISO 3166-1 alpha-2."""
    sales_tax_number: NotRequired[Nullable[str]]
    automated_sales_tax: NotRequired[bool]
    r"""Whether sales tax is calculated automatically for the company"""
    sales_tax_enabled: NotRequired[bool]
    r"""Whether sales tax is enabled for the company"""
    default_sales_tax: NotRequired[TaxRateTypedDict]
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    fiscal_year_start_month: NotRequired[TheStartMonthOfFiscalYear]
    r"""The start month of fiscal year."""
    company_start_date: NotRequired[date]
    r"""Date when company file was created"""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    tracking_categories_enabled: NotRequired[bool]
    r"""Whether tracking categories are enabled for the company on transactions"""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class CompanyInfo(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    status: Optional[CompanyStatus] = None
    r"""Based on the status some functionality is enabled or disabled."""

    legal_name: Optional[str] = None
    r"""The legal name of the company"""

    country: OptionalNullable[str] = UNSET
    r"""country code according to ISO 3166-1 alpha-2."""

    sales_tax_number: OptionalNullable[str] = UNSET

    automated_sales_tax: Optional[bool] = None
    r"""Whether sales tax is calculated automatically for the company"""

    sales_tax_enabled: Optional[bool] = None
    r"""Whether sales tax is enabled for the company"""

    default_sales_tax: Optional[TaxRate] = None

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    fiscal_year_start_month: Optional[TheStartMonthOfFiscalYear] = None
    r"""The start month of fiscal year."""

    company_start_date: Optional[date] = None
    r"""Date when company file was created"""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    tracking_categories_enabled: Optional[bool] = None
    r"""Whether tracking categories are enabled for the company on transactions"""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "company_name",
            "status",
            "legal_name",
            "country",
            "sales_tax_number",
            "automated_sales_tax",
            "sales_tax_enabled",
            "default_sales_tax",
            "currency",
            "language",
            "fiscal_year_start_month",
            "company_start_date",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_mappings",
            "tracking_categories_enabled",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "company_name",
            "country",
            "sales_tax_number",
            "currency",
            "language",
            "custom_mappings",
            "row_version",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .bankaccount import BankAccount, BankAccountTypedDict
from .customfield import CustomField, CustomFieldTypedDict
from .email import Email, EmailTypedDict
from .employeecompensation import (
    EmployeeCompensation,
    EmployeeCompensationInput,
    EmployeeCompensationInputTypedDict,
    EmployeeCompensationTypedDict,
)
from .employeejob import (
    EmployeeJob,
    EmployeeJobInput,
    EmployeeJobInputTypedDict,
    EmployeeJobTypedDict,
)
from .employmentstatus import EmploymentStatus
from .gender import Gender
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .person import Person, PersonTypedDict
from .person_input import PersonInput, PersonInputTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .sociallink import SocialLink, SocialLinkTypedDict
from .team import Team, TeamTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LeavingReason(str, Enum):
    r"""The reason because the employment ended."""

    DISMISSED = "dismissed"
    RESIGNED = "resigned"
    REDUNDANCY = "redundancy"
    OTHER = "other"


class EmploymentType(str, Enum):
    r"""The type of employment relationship the employee has with the organization."""

    CONTRACTOR = "contractor"
    EMPLOYEE = "employee"
    FREELANCE = "freelance"
    TEMP = "temp"
    INTERNSHIP = "internship"
    OTHER = "other"


class EmploymentSubType(str, Enum):
    r"""The work schedule of the employee."""

    FULL_TIME = "full_time"
    PART_TIME = "part_time"
    HOURLY = "hourly"
    OTHER = "other"
    NOT_SPECIFIED = "not_specified"


class EmploymentRoleTypedDict(TypedDict):
    type: NotRequired[Nullable[EmploymentType]]
    r"""The type of employment relationship the employee has with the organization."""
    sub_type: NotRequired[Nullable[EmploymentSubType]]
    r"""The work schedule of the employee."""


class EmploymentRole(BaseModel):
    type: OptionalNullable[EmploymentType] = UNSET
    r"""The type of employment relationship the employee has with the organization."""

    sub_type: OptionalNullable[EmploymentSubType] = UNSET
    r"""The work schedule of the employee."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "sub_type"]
        nullable_fields = ["type", "sub_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ManagerTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    name: NotRequired[Nullable[str]]
    r"""The name of the manager, often a combination of their first and last names."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    email: NotRequired[Nullable[str]]
    r"""The email address of the manager."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""


class Manager(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the manager, often a combination of their first and last names."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    email: OptionalNullable[str] = UNSET
    r"""The email address of the manager."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "employment_status",
        ]
        nullable_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "employment_status",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ProbationPeriodTypedDict(TypedDict):
    start_date: NotRequired[Nullable[date]]
    r"""The date that the employee started their probation period at the company."""
    end_date: NotRequired[Nullable[date]]
    r"""The date that the employee ended their probation period at the company."""


class ProbationPeriod(BaseModel):
    start_date: OptionalNullable[date] = UNSET
    r"""The date that the employee started their probation period at the company."""

    end_date: OptionalNullable[date] = UNSET
    r"""The date that the employee ended their probation period at the company."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["start_date", "end_date"]
        nullable_fields = ["start_date", "end_date"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EmployeeTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    display_name: NotRequired[Nullable[str]]
    r"""The name used to display the employee, often a combination of their first and last names."""
    preferred_name: NotRequired[Nullable[str]]
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    marital_status: NotRequired[Nullable[str]]
    r"""The marital status of the employee."""
    partner: NotRequired[PersonTypedDict]
    division: NotRequired[Nullable[str]]
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""
    division_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the division this employee belongs to."""
    department: NotRequired[Nullable[str]]
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""
    department_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the department ID this employee belongs to."""
    department_name: NotRequired[Nullable[str]]
    r"""Name of the department this employee belongs to."""
    team: NotRequired[Nullable[TeamTypedDict]]
    r"""The team the person is currently in."""
    company_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the company."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    employment_start_date: NotRequired[Nullable[str]]
    r"""A Start Date is the date that the employee started working at the company"""
    employment_end_date: NotRequired[Nullable[str]]
    r"""An End Date is the date that the employee ended working at the company"""
    leaving_reason: NotRequired[Nullable[LeavingReason]]
    r"""The reason because the employment ended."""
    employee_number: NotRequired[Nullable[str]]
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""
    employment_role: NotRequired[EmploymentRoleTypedDict]
    ethnicity: NotRequired[Nullable[str]]
    r"""The ethnicity of the employee"""
    manager: NotRequired[ManagerTypedDict]
    direct_reports: NotRequired[Nullable[List[str]]]
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    deceased_on: NotRequired[Nullable[date]]
    r"""The date the person deceased."""
    country_of_birth: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    gender: NotRequired[Nullable[Gender]]
    r"""The gender represents the gender identity of a person."""
    pronouns: NotRequired[Nullable[str]]
    r"""The preferred pronouns of the person."""
    preferred_language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    languages: NotRequired[List[Nullable[str]]]
    nationalities: NotRequired[List[Nullable[str]]]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    timezone: NotRequired[Nullable[str]]
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""
    source: NotRequired[Nullable[str]]
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""
    source_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""
    record_url: NotRequired[Nullable[str]]
    jobs: NotRequired[Nullable[List[EmployeeJobTypedDict]]]
    compensations: NotRequired[Nullable[List[EmployeeCompensationTypedDict]]]
    works_remote: NotRequired[Nullable[bool]]
    r"""Indicates if the employee works from a remote location."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    tax_code: NotRequired[Nullable[str]]
    tax_id: NotRequired[Nullable[str]]
    dietary_preference: NotRequired[Nullable[str]]
    r"""Indicate the employee's dietary preference."""
    food_allergies: NotRequired[Nullable[List[str]]]
    r"""Indicate the employee's food allergies."""
    probation_period: NotRequired[ProbationPeriodTypedDict]
    tags: NotRequired[Nullable[List[str]]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Employee(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    display_name: OptionalNullable[str] = UNSET
    r"""The name used to display the employee, often a combination of their first and last names."""

    preferred_name: OptionalNullable[str] = UNSET
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    marital_status: OptionalNullable[str] = UNSET
    r"""The marital status of the employee."""

    partner: Optional[Person] = None

    division: OptionalNullable[str] = UNSET
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""

    division_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the division this employee belongs to."""

    department: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""

    department_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the department ID this employee belongs to."""

    department_name: OptionalNullable[str] = UNSET
    r"""Name of the department this employee belongs to."""

    team: OptionalNullable[Team] = UNSET
    r"""The team the person is currently in."""

    company_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of the company."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    employment_start_date: OptionalNullable[str] = UNSET
    r"""A Start Date is the date that the employee started working at the company"""

    employment_end_date: OptionalNullable[str] = UNSET
    r"""An End Date is the date that the employee ended working at the company"""

    leaving_reason: OptionalNullable[LeavingReason] = UNSET
    r"""The reason because the employment ended."""

    employee_number: OptionalNullable[str] = UNSET
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    employment_role: Optional[EmploymentRole] = None

    ethnicity: OptionalNullable[str] = UNSET
    r"""The ethnicity of the employee"""

    manager: Optional[Manager] = None

    direct_reports: OptionalNullable[List[str]] = UNSET
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    deceased_on: OptionalNullable[date] = UNSET
    r"""The date the person deceased."""

    country_of_birth: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    gender: OptionalNullable[Gender] = UNSET
    r"""The gender represents the gender identity of a person."""

    pronouns: OptionalNullable[str] = UNSET
    r"""The preferred pronouns of the person."""

    preferred_language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    languages: Optional[List[Nullable[str]]] = None

    nationalities: Optional[List[Nullable[str]]] = None

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    timezone: OptionalNullable[str] = UNSET
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""

    source: OptionalNullable[str] = UNSET
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""

    source_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""

    record_url: OptionalNullable[str] = UNSET

    jobs: OptionalNullable[List[EmployeeJob]] = UNSET

    compensations: OptionalNullable[List[EmployeeCompensation]] = UNSET

    works_remote: OptionalNullable[bool] = UNSET
    r"""Indicates if the employee works from a remote location."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    social_links: Optional[List[SocialLink]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    tax_code: OptionalNullable[str] = UNSET

    tax_id: OptionalNullable[str] = UNSET

    dietary_preference: OptionalNullable[str] = UNSET
    r"""Indicate the employee's dietary preference."""

    food_allergies: OptionalNullable[List[str]] = UNSET
    r"""Indicate the employee's food allergies."""

    probation_period: Optional[ProbationPeriod] = None

    tags: OptionalNullable[List[str]] = UNSET

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "downstream_id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "partner",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "employment_role",
            "ethnicity",
            "manager",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "languages",
            "nationalities",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_fields",
            "social_links",
            "bank_accounts",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "probation_period",
            "tags",
            "custom_mappings",
            "row_version",
            "deleted",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "id",
            "downstream_id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "ethnicity",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "tags",
            "custom_mappings",
            "row_version",
            "deleted",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EmployeeInputTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    display_name: NotRequired[Nullable[str]]
    r"""The name used to display the employee, often a combination of their first and last names."""
    preferred_name: NotRequired[Nullable[str]]
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    marital_status: NotRequired[Nullable[str]]
    r"""The marital status of the employee."""
    partner: NotRequired[PersonInputTypedDict]
    division: NotRequired[Nullable[str]]
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""
    division_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the division this employee belongs to."""
    department: NotRequired[Nullable[str]]
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""
    department_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the department ID this employee belongs to."""
    department_name: NotRequired[Nullable[str]]
    r"""Name of the department this employee belongs to."""
    team: NotRequired[Nullable[TeamTypedDict]]
    r"""The team the person is currently in."""
    company_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the company."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    employment_start_date: NotRequired[Nullable[str]]
    r"""A Start Date is the date that the employee started working at the company"""
    employment_end_date: NotRequired[Nullable[str]]
    r"""An End Date is the date that the employee ended working at the company"""
    leaving_reason: NotRequired[Nullable[LeavingReason]]
    r"""The reason because the employment ended."""
    employee_number: NotRequired[Nullable[str]]
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""
    employment_status: NotRequired[Nullable[EmploymentStatus]]
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""
    employment_role: NotRequired[EmploymentRoleTypedDict]
    ethnicity: NotRequired[Nullable[str]]
    r"""The ethnicity of the employee"""
    manager: NotRequired[ManagerTypedDict]
    direct_reports: NotRequired[Nullable[List[str]]]
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    deceased_on: NotRequired[Nullable[date]]
    r"""The date the person deceased."""
    country_of_birth: NotRequired[Nullable[str]]
    r"""Country code according to ISO 3166-1 alpha-2."""
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    gender: NotRequired[Nullable[Gender]]
    r"""The gender represents the gender identity of a person."""
    pronouns: NotRequired[Nullable[str]]
    r"""The preferred pronouns of the person."""
    preferred_language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    languages: NotRequired[List[Nullable[str]]]
    nationalities: NotRequired[List[Nullable[str]]]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    timezone: NotRequired[Nullable[str]]
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""
    source: NotRequired[Nullable[str]]
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""
    source_id: NotRequired[Nullable[str]]
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""
    record_url: NotRequired[Nullable[str]]
    jobs: NotRequired[Nullable[List[EmployeeJobInputTypedDict]]]
    compensations: NotRequired[Nullable[List[EmployeeCompensationInputTypedDict]]]
    works_remote: NotRequired[Nullable[bool]]
    r"""Indicates if the employee works from a remote location."""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    bank_accounts: NotRequired[List[BankAccountTypedDict]]
    tax_code: NotRequired[Nullable[str]]
    tax_id: NotRequired[Nullable[str]]
    dietary_preference: NotRequired[Nullable[str]]
    r"""Indicate the employee's dietary preference."""
    food_allergies: NotRequired[Nullable[List[str]]]
    r"""Indicate the employee's food allergies."""
    probation_period: NotRequired[ProbationPeriodTypedDict]
    tags: NotRequired[Nullable[List[str]]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class EmployeeInput(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    display_name: OptionalNullable[str] = UNSET
    r"""The name used to display the employee, often a combination of their first and last names."""

    preferred_name: OptionalNullable[str] = UNSET
    r"""The name the employee prefers to be addressed by, which may be different from their legal name."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    marital_status: OptionalNullable[str] = UNSET
    r"""The marital status of the employee."""

    partner: Optional[PersonInput] = None

    division: OptionalNullable[str] = UNSET
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""

    division_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the division this employee belongs to."""

    department: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""

    department_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the department ID this employee belongs to."""

    department_name: OptionalNullable[str] = UNSET
    r"""Name of the department this employee belongs to."""

    team: OptionalNullable[Team] = UNSET
    r"""The team the person is currently in."""

    company_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of the company."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    employment_start_date: OptionalNullable[str] = UNSET
    r"""A Start Date is the date that the employee started working at the company"""

    employment_end_date: OptionalNullable[str] = UNSET
    r"""An End Date is the date that the employee ended working at the company"""

    leaving_reason: OptionalNullable[LeavingReason] = UNSET
    r"""The reason because the employment ended."""

    employee_number: OptionalNullable[str] = UNSET
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""

    employment_status: OptionalNullable[EmploymentStatus] = UNSET
    r"""The employment status of the employee, indicating whether they are currently employed, inactive, terminated, or in another status."""

    employment_role: Optional[EmploymentRole] = None

    ethnicity: OptionalNullable[str] = UNSET
    r"""The ethnicity of the employee"""

    manager: Optional[Manager] = None

    direct_reports: OptionalNullable[List[str]] = UNSET
    r"""Direct reports is an array of ids that reflect the individuals in an organizational hierarchy who are directly supervised by this specific employee."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    deceased_on: OptionalNullable[date] = UNSET
    r"""The date the person deceased."""

    country_of_birth: OptionalNullable[str] = UNSET
    r"""Country code according to ISO 3166-1 alpha-2."""

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    gender: OptionalNullable[Gender] = UNSET
    r"""The gender represents the gender identity of a person."""

    pronouns: OptionalNullable[str] = UNSET
    r"""The preferred pronouns of the person."""

    preferred_language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    languages: Optional[List[Nullable[str]]] = None

    nationalities: Optional[List[Nullable[str]]] = None

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    timezone: OptionalNullable[str] = UNSET
    r"""The time zone related to the resource. The value is a string containing a standard time zone identifier, e.g. Europe/London."""

    source: OptionalNullable[str] = UNSET
    r"""When the employee is imported as a new hire, this field indicates what system (e.g. the name of the ATS) this employee was imported from."""

    source_id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the employee in the system this employee was imported from (e.g. the ID in the ATS)."""

    record_url: OptionalNullable[str] = UNSET

    jobs: OptionalNullable[List[EmployeeJobInput]] = UNSET

    compensations: OptionalNullable[List[EmployeeCompensationInput]] = UNSET

    works_remote: OptionalNullable[bool] = UNSET
    r"""Indicates if the employee works from a remote location."""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    social_links: Optional[List[SocialLink]] = None

    bank_accounts: Optional[List[BankAccount]] = None

    tax_code: OptionalNullable[str] = UNSET

    tax_id: OptionalNullable[str] = UNSET

    dietary_preference: OptionalNullable[str] = UNSET
    r"""Indicate the employee's dietary preference."""

    food_allergies: OptionalNullable[List[str]] = UNSET
    r"""Indicate the employee's food allergies."""

    probation_period: Optional[ProbationPeriod] = None

    tags: OptionalNullable[List[str]] = UNSET

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "partner",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "employment_role",
            "ethnicity",
            "manager",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "languages",
            "nationalities",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "addresses",
            "phone_numbers",
            "emails",
            "custom_fields",
            "social_links",
            "bank_accounts",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "probation_period",
            "tags",
            "row_version",
            "deleted",
            "pass_through",
        ]
        nullable_fields = [
            "id",
            "first_name",
            "last_name",
            "middle_name",
            "display_name",
            "preferred_name",
            "initials",
            "salutation",
            "title",
            "marital_status",
            "division",
            "division_id",
            "department",
            "department_id",
            "department_name",
            "team",
            "company_id",
            "company_name",
            "employment_start_date",
            "employment_end_date",
            "leaving_reason",
            "employee_number",
            "employment_status",
            "ethnicity",
            "direct_reports",
            "social_security_number",
            "birthday",
            "deceased_on",
            "country_of_birth",
            "description",
            "gender",
            "pronouns",
            "preferred_language",
            "photo_url",
            "timezone",
            "source",
            "source_id",
            "record_url",
            "jobs",
            "compensations",
            "works_remote",
            "tax_code",
            "tax_id",
            "dietary_preference",
            "food_allergies",
            "tags",
            "row_version",
            "deleted",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .deletedepartmentresponse import (
    DeleteDepartmentResponse,
    DeleteDepartmentResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import BaseModel
from apideck_unify.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class HrisDepartmentsDeleteGlobalsTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class HrisDepartmentsDeleteGlobals(BaseModel):
    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class HrisDepartmentsDeleteRequestTypedDict(TypedDict):
    id: str
    r"""ID of the record you are acting upon."""
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""
    service_id: NotRequired[str]
    r"""Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API."""
    raw: NotRequired[bool]
    r"""Include raw response. Mostly used for debugging purposes"""


class HrisDepartmentsDeleteRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""ID of the record you are acting upon."""

    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""

    service_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-service-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API."""

    raw: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""Include raw response. Mostly used for debugging purposes"""


class HrisDepartmentsDeleteResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    delete_department_response: NotRequired[DeleteDepartmentResponseTypedDict]
    r"""Departments"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class HrisDepartmentsDeleteResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    delete_department_response: Optional[DeleteDepartmentResponse] = None
    r"""Departments"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class ProductStatus(str, Enum):
    r"""The current status of the product (active or archived)."""

    ACTIVE = "active"
    ARCHIVED = "archived"


class ImagesTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    url: NotRequired[Nullable[str]]
    r"""The URL of an image of the product."""


class Images(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    url: OptionalNullable[str] = UNSET
    r"""The URL of an image of the product."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "url"]
        nullable_fields = ["id", "url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceProductOptionsTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for the option of the product."""
    name: NotRequired[Nullable[str]]
    r"""The name of the option for the product."""
    values: NotRequired[List[Nullable[str]]]


class EcommerceProductOptions(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the option of the product."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the option for the product."""

    values: Optional[List[Nullable[str]]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "name", "values"]
        nullable_fields = ["id", "name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceProductVariantsOptionsTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for the option of the variant."""
    name: NotRequired[Nullable[str]]
    r"""The name of the option for the variant."""
    value: NotRequired[Nullable[str]]
    r"""The value of the option for the variant."""


class EcommerceProductVariantsOptions(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the option of the variant."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the option for the variant."""

    value: OptionalNullable[str] = UNSET
    r"""The value of the option for the variant."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "name", "value"]
        nullable_fields = ["id", "name", "value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceProductImagesTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    url: NotRequired[Nullable[str]]
    r"""The URL of an image of the variant."""


class EcommerceProductImages(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    url: OptionalNullable[str] = UNSET
    r"""The URL of an image of the variant."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "url"]
        nullable_fields = ["id", "url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class VariantsTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for the variant of the product."""
    name: NotRequired[Nullable[str]]
    r"""The name for the variant, used for displaying to customers."""
    price: NotRequired[Nullable[str]]
    r"""The price of the variant."""
    sku: NotRequired[Nullable[str]]
    r"""The stock keeping unit of the variant."""
    inventory_quantity: NotRequired[Nullable[str]]
    r"""The quantity of the variant in stock."""
    weight: NotRequired[Nullable[str]]
    r"""The weight of the variant."""
    weight_unit: NotRequired[Nullable[str]]
    r"""The unit of measurement for the weight of the variant."""
    options: NotRequired[List[EcommerceProductVariantsOptionsTypedDict]]
    images: NotRequired[List[EcommerceProductImagesTypedDict]]


class Variants(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for the variant of the product."""

    name: OptionalNullable[str] = UNSET
    r"""The name for the variant, used for displaying to customers."""

    price: OptionalNullable[str] = UNSET
    r"""The price of the variant."""

    sku: OptionalNullable[str] = UNSET
    r"""The stock keeping unit of the variant."""

    inventory_quantity: OptionalNullable[str] = UNSET
    r"""The quantity of the variant in stock."""

    weight: OptionalNullable[str] = UNSET
    r"""The weight of the variant."""

    weight_unit: OptionalNullable[str] = UNSET
    r"""The unit of measurement for the weight of the variant."""

    options: Optional[List[EcommerceProductVariantsOptions]] = None

    images: Optional[List[EcommerceProductImages]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "price",
            "sku",
            "inventory_quantity",
            "weight",
            "weight_unit",
            "options",
            "images",
        ]
        nullable_fields = [
            "id",
            "name",
            "price",
            "sku",
            "inventory_quantity",
            "weight",
            "weight_unit",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceProductCategoriesTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    name: NotRequired[Nullable[str]]
    r"""The name of the category."""


class EcommerceProductCategories(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the category."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "name"]
        nullable_fields = ["id", "name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EcommerceProductTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    name: NotRequired[Nullable[str]]
    r"""The name of the product as it should be displayed to customers."""
    description: NotRequired[Nullable[str]]
    r"""A detailed description of the product."""
    status: NotRequired[Nullable[ProductStatus]]
    r"""The current status of the product (active or archived)."""
    price: NotRequired[Nullable[str]]
    r"""The price of the product."""
    sku: NotRequired[Nullable[str]]
    r"""The stock keeping unit of the product."""
    inventory_quantity: NotRequired[Nullable[str]]
    r"""The quantity of the product in stock."""
    images: NotRequired[Nullable[List[ImagesTypedDict]]]
    r"""An array of image URLs for the product."""
    weight: NotRequired[Nullable[str]]
    r"""The weight of the product."""
    weight_unit: NotRequired[Nullable[str]]
    r"""The unit of measurement for the weight of the product."""
    options: NotRequired[List[EcommerceProductOptionsTypedDict]]
    r"""An array of options for the product."""
    variants: NotRequired[List[VariantsTypedDict]]
    tags: NotRequired[List[Nullable[str]]]
    r"""An array of tags for the product, used for organization and searching."""
    categories: NotRequired[List[EcommerceProductCategoriesTypedDict]]
    r"""An array of categories for the product, used for organization and searching."""
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""


class EcommerceProduct(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the product as it should be displayed to customers."""

    description: OptionalNullable[str] = UNSET
    r"""A detailed description of the product."""

    status: OptionalNullable[ProductStatus] = UNSET
    r"""The current status of the product (active or archived)."""

    price: OptionalNullable[str] = UNSET
    r"""The price of the product."""

    sku: OptionalNullable[str] = UNSET
    r"""The stock keeping unit of the product."""

    inventory_quantity: OptionalNullable[str] = UNSET
    r"""The quantity of the product in stock."""

    images: OptionalNullable[List[Images]] = UNSET
    r"""An array of image URLs for the product."""

    weight: OptionalNullable[str] = UNSET
    r"""The weight of the product."""

    weight_unit: OptionalNullable[str] = UNSET
    r"""The unit of measurement for the weight of the product."""

    options: Optional[List[EcommerceProductOptions]] = None
    r"""An array of options for the product."""

    variants: Optional[List[Variants]] = None

    tags: Optional[List[Nullable[str]]] = None
    r"""An array of tags for the product, used for organization and searching."""

    categories: Optional[List[EcommerceProductCategories]] = None
    r"""An array of categories for the product, used for organization and searching."""

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "description",
            "status",
            "price",
            "sku",
            "inventory_quantity",
            "images",
            "weight",
            "weight_unit",
            "options",
            "variants",
            "tags",
            "categories",
            "custom_mappings",
            "created_at",
            "updated_at",
        ]
        nullable_fields = [
            "name",
            "description",
            "status",
            "price",
            "sku",
            "inventory_quantity",
            "images",
            "weight",
            "weight_unit",
            "custom_mappings",
            "created_at",
            "updated_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .connectorsfilter import ConnectorsFilter, ConnectorsFilterTypedDict
from .getconnectorsresponse import GetConnectorsResponse, GetConnectorsResponseTypedDict
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from apideck_unify.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ConnectorConnectorsAllGlobalsTypedDict(TypedDict):
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class ConnectorConnectorsAllGlobals(BaseModel):
    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class ConnectorConnectorsAllRequestTypedDict(TypedDict):
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""
    cursor: NotRequired[Nullable[str]]
    r"""Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response."""
    limit: NotRequired[int]
    r"""Number of results to return. Minimum 1, Maximum 200, Default 20"""
    filter_: NotRequired[ConnectorsFilterTypedDict]
    r"""Apply filters"""


class ConnectorConnectorsAllRequest(BaseModel):
    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""

    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""Number of results to return. Minimum 1, Maximum 200, Default 20"""

    filter_: Annotated[
        Optional[ConnectorsFilter],
        pydantic.Field(alias="filter"),
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = None
    r"""Apply filters"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["appId", "cursor", "limit", "filter"]
        nullable_fields = ["cursor"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ConnectorConnectorsAllResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    get_connectors_response: NotRequired[GetConnectorsResponseTypedDict]
    r"""Connectors"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class ConnectorConnectorsAllResponse(BaseModel):
    next: Callable[[], Optional[ConnectorConnectorsAllResponse]]

    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    get_connectors_response: Optional[GetConnectorsResponse] = None
    r"""Connectors"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.activities import Activities
from apideck_unify.companies import Companies
from apideck_unify.contacts import Contacts
from apideck_unify.customobjects import CustomObjects
from apideck_unify.customobjectschemas import CustomObjectSchemas
from apideck_unify.leads import Leads
from apideck_unify.notes import Notes
from apideck_unify.opportunities import Opportunities
from apideck_unify.pipelines import Pipelines
from apideck_unify.users import Users


class Crm(BaseSDK):
    companies: Companies
    contacts: Contacts
    opportunities: Opportunities
    leads: Leads
    pipelines: Pipelines
    notes: Notes
    users: Users
    activities: Activities
    custom_object_schemas: CustomObjectSchemas
    custom_objects: CustomObjects

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.companies = Companies(self.sdk_configuration)
        self.contacts = Contacts(self.sdk_configuration)
        self.opportunities = Opportunities(self.sdk_configuration)
        self.leads = Leads(self.sdk_configuration)
        self.pipelines = Pipelines(self.sdk_configuration)
        self.notes = Notes(self.sdk_configuration)
        self.users = Users(self.sdk_configuration)
        self.activities = Activities(self.sdk_configuration)
        self.custom_object_schemas = CustomObjectSchemas(self.sdk_configuration)
        self.custom_objects = CustomObjects(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class AssigneeInputTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""


class AssigneeInput(BaseModel):
    id: str
    r"""A unique identifier for an object."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createconsumerrequest import CreateConsumerRequest, CreateConsumerRequestTypedDict
from .createconsumerresponse import (
    CreateConsumerResponse,
    CreateConsumerResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class VaultConsumersAddGlobalsTypedDict(TypedDict):
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class VaultConsumersAddGlobals(BaseModel):
    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class VaultConsumersAddRequestTypedDict(TypedDict):
    create_consumer_request: CreateConsumerRequestTypedDict
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class VaultConsumersAddRequest(BaseModel):
    create_consumer_request: Annotated[
        CreateConsumerRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class VaultConsumersAddResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    create_consumer_response: NotRequired[CreateConsumerResponseTypedDict]
    r"""Consumer created"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class VaultConsumersAddResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    create_consumer_response: Optional[CreateConsumerResponse] = None
    r"""Consumer created"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""
